/*
	netLib.h - Header file for netLib Network Communication Library
	v0.3 - 2007.06.26
	Coded by Ian Seyler (iseyler@gmail.com)
	http://bubbai.ath.cx/projects/netlib/
*/


#include <stdio.h> // fprintf
#include <stdlib.h> // malloc
#include <string.h> // memset
#ifdef MS_WINDOWS
	#include <winsock.h>
	#include <winerror.h>
	typedef int socklen_t; // Windows does not define this.
#endif
#ifdef UNIX
	#include <unistd.h>
	#include <errno.h> // to look at error return values
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <netdb.h> // for dns lookup
#endif
#ifdef BEOS
	#include <unistd.h>
	#include <errno.h> // to look at error return values
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <netdb.h> // for dns lookup
	typedef int socklen_t; // BeOS does not define this.
	#define PF_INET AF_INET // Just using the value for AF_INET (Mac OS X does the same thing)
#endif

/* Structure of a netSocket */
struct netSocketStruct {
	int number; // the socket descriptor from the socket() call
	int inuse; // 1 for active, 0 for not active
	int retval; // all functions will update this variable with the function return value
	struct timeval timeout; // timeout value
	struct sockaddr_in addr; // connector's address information
};

typedef struct netSocketStruct netSocket;

/* Structure of a netBuffer */
struct netBufferStruct {
	int size; // size in bytes of data
	int at; // location in data where we are currently when reading and writing
	char complete; // set to 1 when we have completed the buffer
	char *data; // pointer to the data
};

typedef struct netBufferStruct netBuffer;

#define netOK 0 // For use in comparisons. ie: if (netConnect(tempsock, "blah.com", 80) == netOK)

int netInit (); // initiate the library
int netStop (); // stop the library

netSocket* netInitSocket (); // initialize a new netSocket
netBuffer* netInitBuffer (int size); //initialize a new netBuffer of a specified size in bytes

int netConnect (netSocket *netSock, char *address, unsigned short port); // connect to a netSocket
int netDisconnect (netSocket *netSock); // disconnect from a netSocket

int netListen (netSocket *netSock, unsigned short port); // open a netSocket and listen on it
int netAccept (netSocket *listenSock, netSocket *netSock); // accept a connection from a listening netSocket

int netIsDataPending (netSocket *netSock); // check to see if a netSocket has data that is pending to be received

int netSend (netSocket *netSock, char *data, int bytes); // send data to a netSocket
int netRecv (netSocket *netSock, char *data, int bytes); // receive data from a netSocket

int netSendBuffer (netSocket *netSock, netBuffer *netBuff); // send data to a netSocket (in buffer form)
int netRecvBuffer (netSocket *netSock, netBuffer *netBuff); // receive data from a netSocket (in buffer form)

int netSetTimeout (netSocket *netSock, float timeoutvalue); // set the timeout value on a netsocket
